@@ -269,4 +269,4 @@ request_signal <- function(context,
                            keep.raw = FALSE,
                            cv)
   data <- .Call(rnng_cv_request, context, data, send_mode, recv_mode, timeout,
-                keep.raw, environment(), cv)
+                keep.raw, cv, environment())
