@@ -272,3 +272,62 @@ unlock <- function(socket) invisible(.Call(rnng_socket_unlock, socket))
 #'
 timed_signal <- function(cv, time, flag = TRUE)
   .External(rnng_timed_signal, cv, time, flag)
+
+# nanonext - Weak References ---------------------------------------------------
+
+#' Weak References
+#'
+#' \code{weakref} creates a new weak reference - a special type of R object that
+#'     associates a Value with a Key. Value is kept alive for as long as Key
+#'     remains reachable (i.e. has yet to be garbage collected), even if Value
+#'     itself no longer has any references.
+#'
+#' @param key a reference object (such as an environment or external pointer).
+#' @param value an object.
+#'
+#' @return For \code{weakref}: a weak reference.
+#'
+#'     For \code{weakref_key} and \code{weakref_value}: the key or value
+#'     associated with the weak reference, or NULL if no longer reachable.
+#'
+#' @examples
+#' k <- new.env()
+#' v <- "value"
+#'
+#' w <- weakref(k, v)
+#' w
+#'
+#' @export
+#'
+weakref <- function(key, value) .Call(rnng_weakref_make, key, value)
+
+#' Weakref Key
+#'
+#' \code{weakref_key} retrieves the key associated with a weak reference.
+#'
+#' @param w a weak reference.
+#'
+#' @examples
+#' key <- weakref_key(w)
+#' identical(key, k)
+#'
+#' @rdname weakref
+#' @export
+#'
+weakref_key <- function(w) .Call(rnng_weakref_key, w)
+
+#' Weakref Value
+#'
+#' \code{weakref_value} retrieves the value associated with a weak reference.
+#'
+#' @examples
+#' value <- weakref_value(w)
+#' identical(value, v)
+#'
+#' rm(v)
+#' weakref_value(w)
+#'
+#' @rdname weakref
+#' @export
+#'
+weakref_value <- function(w) .Call(rnng_weakref_value, w)
