@@ -381,26 +381,6 @@ SEXP nano_decode(unsigned char *buf, size_t sz, const int mod, const int kpr) {
 
 // finalizers ------------------------------------------------------------------
 
-void dialer_finalizer(SEXP xptr) {
-
-  if (R_ExternalPtrAddr(xptr) == NULL)
-    return;
-  nng_dialer *xp = (nng_dialer *) R_ExternalPtrAddr(xptr);
-  nng_dialer_close(*xp);
-  R_Free(xp);
-
-}
-
-void listener_finalizer(SEXP xptr) {
-
-  if (R_ExternalPtrAddr(xptr) == NULL)
-    return;
-  nng_listener *xp = (nng_listener *) R_ExternalPtrAddr(xptr);
-  nng_listener_close(*xp);
-  R_Free(xp);
-
-}
-
 static void context_finalizer(SEXP xptr) {
 
   if (R_ExternalPtrAddr(xptr) == NULL)
@@ -484,176 +464,6 @@ SEXP rnng_ctx_close(SEXP context) {
 
 }
 
-// dialers and listeners -------------------------------------------------------
-
-SEXP rnng_dial(SEXP socket, SEXP url, SEXP autostart, SEXP error) {
-
-  if (R_ExternalPtrTag(socket) != nano_SocketSymbol)
-    Rf_error("'socket' is not a valid Socket");
-  nng_socket *sock = (nng_socket *) R_ExternalPtrAddr(socket);
-  const int start = LOGICAL(autostart)[0];
-  const char *up = CHAR(STRING_ELT(url, 0));
-  nng_dialer *dp = R_Calloc(1, nng_dialer);
-  SEXP dialer, klass, attr, newattr;
-
-  int xc;
-  switch (start) {
-  case 0:
-    xc = nng_dialer_create(dp, *sock, up);
-    break;
-  case 1:
-    xc = nng_dial(*sock, up, dp, NNG_FLAG_NONBLOCK);
-    break;
-  default:
-    xc = nng_dial(*sock, up, dp, 0);
-  }
-
-  if (xc) {
-    R_Free(dp);
-    if (LOGICAL(error)[0]) ERROR_OUT(xc);
-    ERROR_RET(xc);
-  }
-
-  PROTECT(dialer = R_MakeExternalPtr(dp, nano_DialerSymbol, R_NilValue));
-  R_RegisterCFinalizerEx(dialer, dialer_finalizer, TRUE);
-
-  PROTECT(klass = Rf_allocVector(STRSXP, 2));
-  SET_STRING_ELT(klass, 0, Rf_mkChar("nanoDialer"));
-  SET_STRING_ELT(klass, 1, Rf_mkChar("nano"));
-  Rf_classgets(dialer, klass);
-  Rf_setAttrib(dialer, nano_IdSymbol, Rf_ScalarInteger((int) dp->id));
-  Rf_setAttrib(dialer, nano_UrlSymbol, url);
-  if (start)
-    Rf_setAttrib(dialer, nano_StateSymbol, Rf_mkString("started"));
-  else
-    Rf_setAttrib(dialer, nano_StateSymbol, Rf_mkString("not started"));
-  Rf_setAttrib(dialer, nano_SocketSymbol, Rf_ScalarInteger((int) sock->id));
-
-  attr = Rf_getAttrib(socket, nano_DialerSymbol);
-  if (attr == R_NilValue) {
-    PROTECT(newattr = Rf_allocVector(VECSXP, 1));
-    SET_VECTOR_ELT(newattr, 0, dialer);
-  } else {
-    R_xlen_t xlen = Rf_xlength(attr);
-    PROTECT(newattr = Rf_allocVector(VECSXP, xlen + 1));
-    for (R_xlen_t i = 0; i < xlen; i++)
-      SET_VECTOR_ELT(newattr, i, VECTOR_ELT(attr, i));
-    SET_VECTOR_ELT(newattr, xlen, dialer);
-  }
-  Rf_setAttrib(socket, nano_DialerSymbol, newattr);
-
-  UNPROTECT(3);
-  return nano_success;
-
-}
-
-SEXP rnng_listen(SEXP socket, SEXP url, SEXP autostart, SEXP error) {
-
-  if (R_ExternalPtrTag(socket) != nano_SocketSymbol)
-    Rf_error("'socket' is not a valid Socket");
-  nng_socket *sock = (nng_socket *) R_ExternalPtrAddr(socket);
-  const int start = LOGICAL(autostart)[0];
-  const char *up = CHAR(STRING_ELT(url, 0));
-  nng_listener *lp = R_Calloc(1, nng_listener);
-  SEXP listener, klass, attr, newattr;
-
-  const int xc = start ? nng_listen(*sock, up, lp, 0) : nng_listener_create(lp, *sock, up);
-  if (xc) {
-    R_Free(lp);
-    if (LOGICAL(error)[0]) ERROR_OUT(xc);
-    ERROR_RET(xc);
-  }
-
-  PROTECT(listener = R_MakeExternalPtr(lp, nano_ListenerSymbol, R_NilValue));
-  R_RegisterCFinalizerEx(listener, listener_finalizer, TRUE);
-
-  PROTECT(klass = Rf_allocVector(STRSXP, 2));
-  SET_STRING_ELT(klass, 0, Rf_mkChar("nanoListener"));
-  SET_STRING_ELT(klass, 1, Rf_mkChar("nano"));
-  Rf_classgets(listener, klass);
-  Rf_setAttrib(listener, nano_IdSymbol, Rf_ScalarInteger((int) lp->id));
-  Rf_setAttrib(listener, nano_UrlSymbol, url);
-  if (start)
-    Rf_setAttrib(listener, nano_StateSymbol, Rf_mkString("started"));
-  else
-    Rf_setAttrib(listener, nano_StateSymbol, Rf_mkString("not started"));
-  Rf_setAttrib(listener, nano_SocketSymbol, Rf_ScalarInteger((int) sock->id));
-
-  attr = Rf_getAttrib(socket, nano_ListenerSymbol);
-  if (attr == R_NilValue) {
-    PROTECT(newattr = Rf_allocVector(VECSXP, 1));
-    SET_VECTOR_ELT(newattr, 0, listener);
-  } else {
-    R_xlen_t xlen = Rf_xlength(attr);
-    PROTECT(newattr = Rf_allocVector(VECSXP, xlen + 1));
-    for (R_xlen_t i = 0; i < xlen; i++)
-      SET_VECTOR_ELT(newattr, i, VECTOR_ELT(attr, i));
-    SET_VECTOR_ELT(newattr, xlen, listener);
-  }
-  Rf_setAttrib(socket, nano_ListenerSymbol, newattr);
-
-  UNPROTECT(3);
-  return nano_success;
-
-}
-
-SEXP rnng_dialer_start(SEXP dialer, SEXP async) {
-
-  if (R_ExternalPtrTag(dialer) != nano_DialerSymbol)
-    Rf_error("'dialer' is not a valid Dialer");
-  nng_dialer *dial = (nng_dialer *) R_ExternalPtrAddr(dialer);
-  const int flags = LOGICAL(async)[0] ? NNG_FLAG_NONBLOCK : 0;
-  const int xc = nng_dialer_start(*dial, flags);
-  if (xc)
-    ERROR_RET(xc);
-
-  Rf_setAttrib(dialer, nano_StateSymbol, Rf_mkString("started"));
-  return nano_success;
-
-}
-
-SEXP rnng_listener_start(SEXP listener) {
-
-  if (R_ExternalPtrTag(listener) != nano_ListenerSymbol)
-    Rf_error("'listener' is not a valid Listener");
-  nng_listener *list = (nng_listener *) R_ExternalPtrAddr(listener);
-  const int xc = nng_listener_start(*list, 0);
-  if (xc)
-    ERROR_RET(xc);
-
-  Rf_setAttrib(listener, nano_StateSymbol, Rf_mkString("started"));
-  return nano_success;
-
-}
-
-SEXP rnng_dialer_close(SEXP dialer) {
-
-  if (R_ExternalPtrTag(dialer) != nano_DialerSymbol)
-    Rf_error("'dialer' is not a valid Dialer");
-  nng_dialer *dial = (nng_dialer *) R_ExternalPtrAddr(dialer);
-  const int xc = nng_dialer_close(*dial);
-  if (xc)
-    ERROR_RET(xc);
-
-  Rf_setAttrib(dialer, nano_StateSymbol, Rf_mkString("closed"));
-  return nano_success;
-
-}
-
-SEXP rnng_listener_close(SEXP listener) {
-
-  if (R_ExternalPtrTag(listener) != nano_ListenerSymbol)
-    Rf_error("'listener' is not a valid Listener");
-  nng_listener *list = (nng_listener *) R_ExternalPtrAddr(listener);
-  const int xc = nng_listener_close(*list);
-  if (xc)
-    ERROR_RET(xc);
-
-  Rf_setAttrib(listener, nano_StateSymbol, Rf_mkString("closed"));
-  return nano_success;
-
-}
-
 // send and recv ---------------------------------------------------------------
 
 SEXP rnng_send(SEXP con, SEXP data, SEXP mode, SEXP block) {
@@ -1323,3 +1133,47 @@ SEXP rnng_stats_get(SEXP object, SEXP stat) {
   return out;
 
 }
+
+// terminate the library -------------------------------------------------------
+
+SEXP rnng_fini(void) {
+
+  nng_fini();
+  return R_NilValue;
+
+}
+
+// weakref ---------------------------------------------------------------------
+
+SEXP rnng_weakref_make(SEXP key, SEXP value) {
+
+  return R_MakeWeakRef(key, value, R_NilValue, FALSE);
+
+}
+
+SEXP rnng_weakref_key(SEXP w) {
+
+  return R_WeakRefKey(w);
+
+}
+
+SEXP rnng_weakref_value(SEXP w) {
+
+  return R_WeakRefValue(w);
+
+}
+
+SEXP rnng_strcat(SEXP a, SEXP b) {
+
+  const char *ap = CHAR(STRING_ELT(a, 0));
+  const char *bp = CHAR(STRING_ELT(b, 0));
+  R_xlen_t alen = Rf_xlength(STRING_ELT(a, 0));
+  R_xlen_t blen = Rf_xlength(STRING_ELT(b, 0));
+
+  char *out = R_alloc(sizeof(char), alen + blen + 1);
+  memcpy(out, ap, alen);
+  memcpy(out + alen, bp, blen + 1);
+
+  return Rf_mkString(out);
+
+}
