@@ -159,14 +159,15 @@ static const R_CallMethodDef callMethods[] = {
   {"rnng_cv_value", (DL_FUNC) &rnng_cv_value, 1},
   {"rnng_cv_wait", (DL_FUNC) &rnng_cv_wait, 1},
   {"rnng_device", (DL_FUNC) &rnng_device, 2},
-  {"rnng_dial", (DL_FUNC) &rnng_dial, 4},
+  {"rnng_dial", (DL_FUNC) &rnng_dial, 5},
   {"rnng_dialer_close", (DL_FUNC) &rnng_dialer_close, 1},
   {"rnng_dialer_start", (DL_FUNC) &rnng_dialer_start, 2},
   {"rnng_base64dec", (DL_FUNC) &rnng_base64dec, 2},
   {"rnng_base64enc", (DL_FUNC) &rnng_base64enc, 2},
+  {"rnng_fini", (DL_FUNC) &rnng_fini, 0},
   {"rnng_get_opt", (DL_FUNC) &rnng_get_opt, 2},
   {"rnng_is_nul_byte", (DL_FUNC) &rnng_is_nul_byte, 1},
-  {"rnng_listen", (DL_FUNC) &rnng_listen, 4},
+  {"rnng_listen", (DL_FUNC) &rnng_listen, 5},
   {"rnng_listener_close", (DL_FUNC) &rnng_listener_close, 1},
   {"rnng_listener_start", (DL_FUNC) &rnng_listener_start, 1},
   {"rnng_messenger", (DL_FUNC) &rnng_messenger, 1},
@@ -198,6 +199,7 @@ static const R_CallMethodDef callMethods[] = {
   {"rnng_stream_close", (DL_FUNC) &rnng_stream_close, 1},
   {"rnng_stream_dial", (DL_FUNC) &rnng_stream_dial, 3},
   {"rnng_stream_listen", (DL_FUNC) &rnng_stream_listen, 3},
+  {"rnng_strcat", (DL_FUNC) &rnng_strcat, 2},
   {"rnng_strerror", (DL_FUNC) &rnng_strerror, 1},
   {"rnng_subscribe", (DL_FUNC) &rnng_subscribe, 3},
   {"rnng_tls_config", (DL_FUNC) &rnng_tls_config, 4},
@@ -205,6 +207,10 @@ static const R_CallMethodDef callMethods[] = {
   {"rnng_unresolved2", (DL_FUNC) &rnng_unresolved2, 1},
   {"rnng_url_parse", (DL_FUNC) &rnng_url_parse, 1},
   {"rnng_version", (DL_FUNC) &rnng_version, 0},
+  {"rnng_weakref_make", (DL_FUNC) &rnng_weakref_make, 2},
+  {"rnng_weakref_key", (DL_FUNC) &rnng_weakref_key, 1},
+  {"rnng_weakref_value", (DL_FUNC) &rnng_weakref_value, 1},
+  {"rnng_write_cert", (DL_FUNC) &rnng_write_cert, 3},
   {NULL, NULL, 0}
 };
 
