@@ -151,7 +151,8 @@ nanotest(is.double(stat(n$dialer[[1L]], "id")))
 nanotestz(n$listen(url = "inproc://three", autostart = FALSE))
 nanotestz(n$listener_start())
 nanotest(inherits(n$listener[[2L]], "nanoListener"))
-nanotest(is.double(stat(n$listener[[1L]], "id")))
+nanotest(is.double(stat(n$listener[[2L]], "id")))
+nanotestz(n$dial(url = "inproc://four"))
 nanotestz(close(n$listener[[1]]))
 nanotestw(close(n$listener[[1]]) == 12L)
 nanotestz(close(n1$dialer[[1]]))
@@ -220,6 +221,7 @@ nanotestn(cv_reset(cv))
 nanotestz(cv_value(cv))
 nanotestnn(cr <- request_signal(req$context, "test", timeout = 500, keep.raw = TRUE, cv = cv))
 nanotestnn(cr <- recv_aio_signal(rep, timeout = 500, keep.raw = TRUE, cv = cv))
+nanotestnn(call_aio(cr)$raw)
 nanotestnn(cr <- recv_aio_signal(context(rep), timeout = 500, cv = cv))
 nanotestz(pipe_notify(rep, cv))
 nanotestz(pipe_notify(req$socket, cv = cv, add = FALSE, remove = TRUE, flag = FALSE))
@@ -316,18 +318,18 @@ nanotestnn(ncurl("http://www.cam.ac.uk/"))
 nanotestnn(ncurl("http://www.cam.ac.uk/", follow = FALSE, response = "date"))
 nanotestnn(ncurl("http://www.cam.ac.uk/", follow = FALSE, response = list("date")))
 nanotestnn(ncurl("http://www.cam.ac.uk/", follow = TRUE))
-nanotestnn(ncurl("http://httpbin.org/post", convert = FALSE, method = "POST", headers = c(`Content-Type` = "text/plain"), data = "test", response = "server", timeout = 3000L))
-nanotestnn(ncurl("http://httpbin.org/post", convert = FALSE, method = "POST", headers = list(`Content-Type` = "text/plain"), response = list("Date", "Server"), timeout = 3000))
+nanotestnn(ncurl("http://postman-echo.com/post", convert = FALSE, method = "POST", headers = c(`Content-Type` = "text/plain"), data = "test", response = "server", timeout = 3000L))
+nanotestnn(ncurl("http://postman-echo.com/post", convert = FALSE, method = "POST", headers = list(`Content-Type` = "text/plain"), response = list("Date", "Server"), timeout = 3000))
 relo <- ncurl("https://cran.r-project.org/package=nanonext", convert = FALSE, response = "server", async = TRUE)
 nanotest(is.integer(call_aio(relo)$status))
 relo <- ncurl("https://cran.r-project.org/package=nanonext", convert = FALSE, response = list("server"), async = TRUE)
 nanotest(is.integer(call_aio(relo)$status))
-put1 <- ncurl("http://httpbin.org/put", async = TRUE, method = "PUT", headers = c(Authorization = "Bearer token"), data = "test", response = c("date", "server"), timeout = 3000L)
+put1 <- ncurl("http://postman-echo.com/put", async = TRUE, method = "PUT", headers = c(Authorization = "Bearer token"), data = "test", response = c("date", "server"), timeout = 3000L)
 nanotest(is.integer(call_aio(put1)$status))
 nanotestnn(put1$headers)
 nanotestnn(put1$raw)
 nanotestnn(put1$data)
-put2 <- ncurl("http://httpbin.org/put", async = TRUE, method = "PUT", headers = list(Authorization = "Bearer token"), response = list("date", "server"), timeout = 3000L)
+put2 <- ncurl("http://postman-echo.com/put", async = TRUE, method = "PUT", headers = list(Authorization = "Bearer token"), response = list("date", "server"), timeout = 3000L)
 nanotestp(put2)
 nanotest(is.integer(call_aio(put2)$status))
 nanotest(is.null(stop_aio(put2)))
@@ -336,10 +338,10 @@ nanotest(is_error_value(ncurl("http", async = TRUE)$data))
 haio <- ncurl("https://i.i", async = TRUE)
 nanotest(is_error_value(call_aio(haio)$raw))
 nanotestp(haio$data)
-sess <- ncurl_session("https://httpbin.org/post", method = "POST", headers = c(`Content-Type` = "text/plain"), data = "test", response = "server", timeout = 3000L)
+sess <- ncurl_session("https://postman-echo.com/post", method = "POST", headers = c(`Content-Type` = "text/plain"), data = "test", response = "server", timeout = 3000L)
 nanotestnn(sess)
 nanotest(is_error_value(sess) || length(transact(sess)) == 4L)
-sess <- ncurl_session("https://httpbin.org/post", convert = FALSE, method = "POST", headers = list(`Content-Type` = "text/plain"), response = list("Date", "Server"), timeout = 3000)
+sess <- ncurl_session("https://postman-echo.com/post", convert = FALSE, method = "POST", headers = list(`Content-Type` = "text/plain"), response = list("Date", "Server"), timeout = 3000)
 nanotestnn(sess)
 nanotest(is_error_value(sess) || length(transact(sess)) == 4L)
 nanotest(is_error_value(sess) || close(sess) == 0L)
@@ -495,22 +497,27 @@ nanotesterr(tls_config(server = file), "Cryptographic error")
 nanotesterr(tls_config(client = c(pem, pem)), "Cryptographic error")
 nanotesterr(tls_config(server = c(pem, pem)), "Cryptographic error")
 unlink(file)
-nanotestxp(tls <- tls_config(client = c(
-"-----BEGIN CERTIFICATE-----
-MIICGTCCAZ+gAwIBAgIQCeCTZaz32ci5PhwLBCou8zAKBggqhkjOPQQDAzBOMQswCQYDVQQGEwJV
-UzEXMBUGA1UEChMORGlnaUNlcnQsIEluYy4xJjAkBgNVBAMTHURpZ2lDZXJ0IFRMUyBFQ0MgUDM4
-NCBSb290IEc1MB4XDTIxMDExNTAwMDAwMFoXDTQ2MDExNDIzNTk1OVowTjELMAkGA1UEBhMCVVMx
-FzAVBgNVBAoTDkRpZ2lDZXJ0LCBJbmMuMSYwJAYDVQQDEx1EaWdpQ2VydCBUTFMgRUNDIFAzODQg
-Um9vdCBHNTB2MBAGByqGSM49AgEGBSuBBAAiA2IABMFEoc8Rl1Ca3iOCNQfN0MsYndLxf3c1Tzvd
-lHJS7cI7+Oz6e2tYIOyZrsn8aLN1udsJ7MgT9U7GCh1mMEy7H0cKPGEQQil8pQgO4CLp0zVozptj
-n4S1mU1YoI71VOeVyaNCMEAwHQYDVR0OBBYEFMFRRVBZqz7nLFr6ICISB4CIfBFqMA4GA1UdDwEB
-/wQEAwIBhjAPBgNVHRMBAf8EBTADAQH/MAoGCCqGSM49BAMDA2gAMGUCMQCJao1H5+z8blUD2Wds
-Jk6Dxv3J+ysTvLd6jLRl0mlpYxNjOyZQLgGheQaRnUi/wr4CMEfDFXuxoJGZSZOoPHzoRgaLLPIx
-AJSdYsiJvRmEFOml+wG4DXZDjC5Ty3zfDBeWUA==
------END CERTIFICATE-----", ""
-)))
+cert <- write_cert(cn = "127.0.0.1")
+nanotest(is.list(cert) && length(cert) == 2L)
+nanotest(identical(names(cert), c("server", "client")))
+nanotestxp(tls <- tls_config(client = cert$client))
 nanotest(inherits(tls, "tlsConfig"))
 nanotestp(tls)
-nanotest(is_error_value(ncurl("https://www.cam.ac.uk/", tls = tls)$status))
-nanotest(is_error_value(call_aio(ncurl("https://www.cam.ac.uk/", async = TRUE, tls = tls))$status))
+nanotest(is_error_value(ncurl("https://www.r-project.org/", tls = tls)$status))
+nanotest(is_error_value(call_aio(ncurl("https://www.r-project.org/", async = TRUE, tls = tls))$status))
+nanotestxp(s <- socket(listen = "tls+tcp://127.0.0.1:5556", tls = tls_config(server = cert$server)))
+nanotestxp(s1 <- socket(dial = "tls+tcp://127.0.0.1:5556", tls = tls))
+nanotestw(dial(s, url = "tls+tcp://[", tls = tls, error = FALSE) > 0)
+nanotestw(listen(s, url = "tls+tcp://[", tls = tls, error = FALSE) > 0)
+close(s1)
+close(s)
 nanotestxp(tls <- tls_config())
+nanotestxp(s <- socket(listen = "wss://127.0.0.1:5557", tls = tls))
+nanotestz(close(s))
+
+k <- new.env()
+v <- "test"
+nanotest(typeof(w <- weakref(k, v)) == "weakref")
+nanotest(identical(weakref_key(w), k))
+nanotest(identical(weakref_value(w), v))
+nanotest(strcat("hello ", "world") == "hello world")
